/*
 * * Copyright (C) 2009-2011 Ali <aliov@xfce.org>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#ifdef XFCE_DISABLE_DEPRECATED
#undef XFCE_DISABLE_DEPRECATED
#endif
#include <libxfce4util/libxfce4util.h>
#include <libxfce4ui/libxfce4ui.h>

#include "parole-session.h"

static void parole_session_finalize   (GObject *object);

#define PAROLE_SESSION_GET_PRIVATE(o) \
(G_TYPE_INSTANCE_GET_PRIVATE ((o), PAROLE_TYPE_SESSION, ParoleSessionPrivate))

struct ParoleSessionPrivate
{
    XfceSMClient  *client;
    gboolean       managed;
};

enum
{
    SAVE_STATE,
    LAST_SIGNAL
};

static guint signals [LAST_SIGNAL] = { 0 };

G_DEFINE_TYPE (ParoleSession, parole_session, G_TYPE_OBJECT)

static void
parole_session_class_init (ParoleSessionClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    signals [SAVE_STATE] = 
        g_signal_new ("save-state",
                      PAROLE_TYPE_SESSION,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET(ParoleSessionClass, save_state),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0, G_TYPE_NONE);

    object_class->finalize = parole_session_finalize;

    g_type_class_add_private (klass, sizeof (ParoleSessionPrivate));
}

static void
parole_session_init (ParoleSession *session)
{
    session->priv = PAROLE_SESSION_GET_PRIVATE (session);
    
    session->priv->client = NULL;
}

static void
parole_session_finalize (GObject *object)
{
    ParoleSession *session;

    session = PAROLE_SESSION (object);
    
    if ( session->priv->client != NULL )
    {
	g_object_unref (session->priv->client);
    }

    G_OBJECT_CLASS (parole_session_parent_class)->finalize (object);
}

ParoleSession *
parole_session_get (const gchar *client_id)
{
    static gpointer parole_session_obj = NULL;
    ParoleSession *session = NULL;
    GError *error = NULL;
    gchar *current_dir;

    const gchar *restart_command[] =
    {
        "parole",
	"--restart",
	NULL
    };

    if ( G_LIKELY (parole_session_obj != NULL ) )
    {
	g_object_ref (parole_session_obj);
    }
    else
    {
        session = g_object_new (PAROLE_TYPE_SESSION, NULL);
        current_dir = g_get_current_dir ();
        session->priv->client = xfce_sm_client_get_full (XFCE_SM_CLIENT_RESTART_NORMAL,
                                                         XFCE_SM_CLIENT_PRIORITY_DEFAULT,
	                                                 client_id,
                                                         current_dir,
                                                         restart_command,
							 PACKAGE_NAME ".desktop");
        g_free (current_dir);
        session->priv->managed = xfce_sm_client_connect (session->priv->client, &error);

        if ( G_UNLIKELY (error) )
        {
            g_warning ("Failed to connect to session manager : %s", error->message);
            g_error_free (error);
        }
	parole_session_obj = session;
	g_object_add_weak_pointer (parole_session_obj, &parole_session_obj);
    }
    
    return PAROLE_SESSION (parole_session_obj);
}
